/*
 * Copyright (C) 2000-2002 Wojciech Purczynski <cliph@isec.pl>
 *
 * THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
 * IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
 * THE FREE SOFTWARE  FOUNDATION;  EITHER  VERSION 2 OF THE LICENSE, OR
 * (AT YOUR OPTION) ANY LATER VERSION.
 *
 * THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED  WARRANTY OF
 * MERCHANTABILITY OR FITNESS  FOR A PARTICULAR  PURPOSE.  SEE THE
 * GNU GENERAL PUBLIC LICENSE FOR MORE DETAILS.
 */

#include "module.h"

#include <linux/config.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/dcache.h>
#include <linux/unistd.h>
#include <linux/ptrace.h>
#include <linux/sched.h>
#include <linux/smp_lock.h>

#include "config.h"
#include "rule.h"
#include "lib.h"
#include "sys.h"
#include "fs.h"

extern void * sys_call_table[];
#define sys_exit(a) ((int (*)(int))sys_call_table[__NR_exit])((a))

rettype (*old_sys_execve)(struct pt_regs);
asmlinkage rettype new_sys_execve(struct pt_regs);

rettype (*old_sys_setresuid)(uid_t, uid_t, uid_t);
#ifdef CONFIG_UID16
rettype (*old_sys_setresuid16)(uid_t, uid_t, uid_t);
#endif /* CONFIG_UID16 */

int check_secure(int enable_secure)
{
	struct vm_area_struct *vma;
	struct dentry * entry=NULL;
	int cantread;

	vma=current->mm->mmap;
	while (vma) {
		if ((vma->vm_flags & VM_EXECUTABLE) && vma->vm_file) {
			entry=vma->vm_file->f_dentry;
			break;
		}
		vma=vma->vm_next;
	}
	
	if (!entry) return -ESRCH;
	cantread=permission(entry->d_inode, MAY_READ);
	
	if (!cantread && !enable_secure)
		return 0; /* do not need to do any extra checks here */
	
	/*
	 * We do not allow to share parts of the process's executable.
	 */
	if (atomic_read(&current->fs->count) > 1 ||
	    atomic_read(&current->sig->count) > 1 ||
	    atomic_read(&current->files->count) > 1)
		return -EPERM;
	
	return 0;
}

asmlinkage rettype new_sys_execve(struct pt_regs regs)
{
	int error=0;
	char * filename;
	const rule_t * rule;
	int enable_secure=1;
	kernel_cap_t old_effective;
	kernel_cap_t old_permitted;
	uid_t old_euid;
	gid_t old_egid;

	MOD_INC_USE_COUNT;
	lock_kernel_2_2();
	
	rule_lock();
	rule=rule_find();
	CHK_PTR(rule);

	/* log */
	filename=get_realpath((char*)regs.ebx);
	CHK_PTR(filename);
	
	error=rule_check_execve(rule, filename);
	
	if (error) log(rule, LOG_SECURITY, "is not permitted to execute %.256s", filename);
	else log(rule, LOG_DEBUG, "is executing %.256s", filename);
	rule_unlock();
	
	put_realpath(filename);

	if (error) goto exit;

	old_euid=current->euid;
	old_egid=current->egid;
	old_effective=current->cap_effective;
	old_permitted=current->cap_permitted;
			
	filename=getname((char *)regs.ebx);
	if (IS_ERR(filename)) {
		error=PTR_ERR(filename);
		goto exit;
	}

	/*
	 * This little hack work-arounds broken capabilities inheritance.
	 * 2.2.x/2.4.x kernels just raise permitted set if real uid is 0.
	 */
	if (!issecure(SECURE_NOROOT) && current->uid==0)
		current->cap_permitted=cap_bset; /* cap_bset is sufficient */

	error=do_execve(filename, (char **)regs.ecx, (char **)regs.edx, &regs);
	putname(filename);
	
	if (error) {
		/* In the case of error we restore old permitted set. */
		current->cap_permitted=old_permitted;
		goto exit;
	}
	
	current->ptrace &= ~PT_DTRACE;
	
        /*
	 * Starting from this point we have to exit with sys_exit()
	 * because new process's structures have been already allocated.
	 */
	
	/* We prevent from dumping core for _all_ binaries. */
#ifdef KERNEL_2_4
	current->mm->dumpable=0;
#else
	current->dumpable=0;
#endif
	
	rule_lock();
	rule=rule_find();
	if (IS_ERR(rule)) {
		error=PTR_ERR(rule);
		goto out;
	}
	
	if (current->pid != 1) rule_set_caps(rule);
		
	/* checking if the process's privileges has changed */
	if ((old_euid != current->euid
	     && !cap_raised(old_effective, CAP_SETUID)) ||
	    (old_egid != current->egid
	     && !cap_raised(old_effective, CAP_SETGID)) ||
	    ((!cap_issubset(current->cap_effective, old_effective) ||
	      !cap_issubset(current->cap_permitted, old_permitted))
	     && !cap_raised(old_effective, CAP_SETPCAP)))
	{
		if (rule->clear_environ) {
			error=clear_environ();
			if (error) goto out;
		}
	} else
		enable_secure = 0;
	
	error=check_secure(enable_secure);
	if (error) goto out;
	
	error=rule_change_rootdir(rule);
	
out:
	rule_unlock();
	if (error) {
		unlock_kernel_2_2();
		MOD_DEC_USE_COUNT;
		sys_exit(-error);
		for (;;); /* NEVER REACHED */
	}
exit:
	unlock_kernel_2_2();
	MOD_DEC_USE_COUNT;
	return error;
} /* new_sys_execve */

asmlinkage rettype new_sys_setresuid(uid_t ruid, uid_t euid, uid_t suid)
{
	const rule_t * rule;
	uid_t old_ruid, old_euid, old_suid;
	kernel_cap_t effective, permitted;
	int error;

	MOD_INC_USE_COUNT;	
	lock_kernel_2_2();
	
	rule_lock();
	rule=rule_find();
	CHK_PTR(rule);
	
	if (rule->check_setuid && (ruid==0 || euid==0 || suid==0) &&
	    (UID!=0 || EUID!=0 || SUID!=0)) {
		log(rule, LOG_SECURITY, "is trying to do setresuid(%u,%u,%u)", ruid, euid, suid);
		rule_unlock();
		error=-EPERM;
		goto exit;
	}

	effective=current->cap_effective;
	permitted=current->cap_permitted;
	old_ruid=current->uid;
	old_euid=current->euid;
	old_suid=current->suid;

	error=old_sys_setresuid(ruid, euid, suid);

	/* it overrides original cap_emulate_setxuid */
	if (!error && !issecure(SECURE_NO_SETUID_FIXUP))
		new_cap_emulate_setxuid(rule, effective, permitted,
					old_ruid, old_euid, old_suid);
	rule_unlock();
	
exit:
	unlock_kernel_2_2();
	MOD_DEC_USE_COUNT;
	return error;
} /* new_sys_setresuid */

#ifdef CONFIG_UID16
#include <linux/highuid.h>
asmlinkage rettype new_sys_setresuid16(old_uid_t ruid, old_uid_t euid, old_uid_t suid)
{
	return new_sys_setresuid(low2highuid(ruid),
	                         low2highuid(euid),
				 low2highuid(suid));
}
#endif /* CONFIG_UID16 */

hook_t arch_hooks[]={
	{ __NR_execve, (void**)&old_sys_execve, new_sys_execve },
#ifdef CONFIG_UID16
	{ __NR_setresuid32, (void**)&old_sys_setresuid, new_sys_setresuid },
	{ __NR_setresuid, (void**)&old_sys_setresuid16, new_sys_setresuid16 },
#else
	{ __NR_setresuid, (void**)&old_sys_setresuid, new_sys_setresuid },
#endif /* CONFIG_UID16 */
	{ 0, NULL, NULL }
};
