/*
 * Copyright (C) 2001 Wojciech Purczynski <cliph@isec.pl>
 *
 * THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
 * IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
 * THE FREE SOFTWARE  FOUNDATION;  EITHER  VERSION 2 OF THE LICENSE, OR
 * (AT YOUR OPTION) ANY LATER VERSION.
 *
 * THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED  WARRANTY OF
 * MERCHANTABILITY OR FITNESS  FOR A PARTICULAR  PURPOSE.  SEE THE
 * GNU GENERAL PUBLIC LICENSE FOR MORE DETAILS.
 */

#ifndef _CAPSEL_RULE_H
#define _CAPSEL_RULE_H

#include "config.h"

#ifndef __KERNEL__

#include <stdint.h>
#include <sys/types.h>
typedef uint32_t __u32;
#define _LINUX_CONFIG_H
#define _LINUX_TYPES_H
#define _LINUX_FS_H
#define __KERNEL__
#include <linux/capability.h>
#undef  __KERNEL__

#endif /* !__KERNEL__ */

/* Raise/lower mask for single capabilities set */
typedef struct {
	kernel_cap_t raise;
	kernel_cap_t lower;
} cap_mask_t;

/* Three masks for three capabilities sets */
typedef struct {
	cap_mask_t effective;
	cap_mask_t permitted;
	cap_mask_t inheritable;
} capabilities_t;

/* List of allowed executables */
typedef struct executable_t {
	char * name;
	struct executable_t * next;
} executable_t;

#define CAP_DEFAULT				\
( (capabilities_t) {				\
	{ CAP_EMPTY_SET, CAP_EMPTY_SET },	\
	{ CAP_EMPTY_SET, CAP_EMPTY_SET },	\
	{ CAP_EMPTY_SET, CAP_EMPTY_SET },	\
} )

#define UIDCAP_DEFAULT				\
( (capabilities_t) {				\
	{ CAP_EMPTY_SET, CAP_FULL_SET },	\
	{ CAP_EMPTY_SET, CAP_FULL_SET },	\
	{ CAP_EMPTY_SET, CAP_FULL_SET },	\
} )

/* Rule definition */
typedef struct rule_t {
	char * name;
	/* Options */
	int 		check_uid	:1;
	int 		check_euid	:1;
	int 		check_gid	:1;
	int 		check_egid	:1;
	unsigned	uid;
	unsigned	euid;
	unsigned	gid;
	unsigned	egid;
	int 		check_groups	:1;
	int		check_execve	:2;
	int		perm_execve	:1;
	int		check_chroot	:1;	/* chkchroot */
	int		check_setuid	:1;	/* chksetuid */
	int		clear_environ	:1;	/* clrenv */
	unsigned 	log_level	:2;
	executable_t 	* executables;		/* list of executables */
	char		* root_dir;		/* root directory */
	capabilities_t 	caps;
	capabilities_t	uidcaps;		/* setuid capabilities mask */
	unsigned short	bind_tcp;		/* bindtcp */
	unsigned short	bind_udp;		/* bindudp */
	struct rule_t * next;
	unsigned	size;			/* size of rule and its members */
} rule_t;

#ifndef TRUE
#define TRUE	-1
#endif

#ifndef FALSE
#define FALSE	0
#endif

/* perm_execve */
#define EXEC_DENY	0
#define EXEC_ALLOW	-1

/* check_execve */
#define EXEC_NONE	0
#define EXEC_LIST	1
#define EXEC_ALL	-1

#define LOG_NOTHING	0
#define LOG_DEBUG	1
#define LOG_SECURITY	2

#define CAP_EFFECTIVE	1
#define CAP_PERMITTED	2
#define CAP_INHERITABLE	4

/* default and secure values */
#define RULE_DEFAULT					\
(rule_t){						\
	NULL,			/* name		 */	\
	FALSE,			/* check_uid 	 */	\
	FALSE,			/* check_euid 	 */	\
	FALSE,			/* check_gid 	 */	\
	FALSE,			/* check_egid 	 */	\
	0,			/* uid 		 */	\
	0,			/* euid		 */	\
	0,			/* gid 		 */	\
	0,			/* egid 	 */	\
	TRUE,			/* check_groups	 */	\
	EXEC_NONE,		/* check_execve  */	\
	EXEC_ALLOW,		/* perm_execve 	 */	\
	TRUE,			/* check_chroot  */	\
	FALSE,			/* check_setuid  */	\
	TRUE,			/* clear_environ */	\
	LOG_SECURITY,		/* log_level	 */	\
	NULL,			/* executables   */	\
	NULL,			/* root_dir      */	\
	CAP_DEFAULT,		/* caps          */	\
	UIDCAP_DEFAULT,		/* uidcaps       */	\
	0,			/* bind_tcp      */	\
	0,			/* bind_udp      */	\
	NULL,			/* next		 */	\
	sizeof(rule_t)		/* size		 */	\
}

#ifdef __KERNEL__
#include <linux/spinlock.h>
#endif

typedef struct {
#ifdef __KERNEL__
	rwlock_t lock;
#endif /* __KERNEL__ */
	unsigned count;		/* number of rules */
	unsigned size;		/* total size of configuration */
	rule_t * first_rule;
	rule_t * last_rule;
	rule_t * default_rule;
} rule_list_t;

#ifdef __KERNEL__
#define EMPTY_RULE_LIST { RW_LOCK_UNLOCKED, 0, 0, NULL, NULL, NULL }
#else
#define EMPTY_RULE_LIST { 0, 0, NULL, NULL, NULL }
#endif

#ifdef __KERNEL__

/* exported variables */
extern rule_list_t rules;
extern rule_list_t new_rules;
extern const rule_t default_rule; 	/* this is global default rule */

const rule_t * rule_find(void);
#define rule_lock() read_lock(&rules.lock)
#define rule_unlock() read_unlock(&rules.lock)

int rule_change_rootdir(const rule_t *);
int rule_check_execve(const rule_t *, const char *);
void rule_set_caps(const rule_t *);
rule_t * rule_copy_from_user(const rule_t *);
void rule_free_all(void);
#endif /* __KERNEL__ */
void rule_free_list(rule_list_t *);
void rule_free(rule_t*);

#endif /* _CAPSEL_RULE_H */
