/*
 * Copyright (C) 2001,2002 Wojciech Purczynski <cliph@isec.pl>
 *
 * THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
 * IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
 * THE FREE SOFTWARE  FOUNDATION;  EITHER  VERSION 2 OF THE LICENSE, OR
 * (AT YOUR OPTION) ANY LATER VERSION.
 *
 * THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED  WARRANTY OF
 * MERCHANTABILITY OR FITNESS  FOR A PARTICULAR  PURPOSE.  SEE THE
 * GNU GENERAL PUBLIC LICENSE FOR MORE DETAILS.
 */

#include "config.h"
#include "ioctl.h"

#include <stdio.h>
#include <fcntl.h>
#include <paths.h>
#include <sys/ioctl.h>
#include <errno.h>

/*
 * Reads version of capsel module.
 * In case of error it sets errno global variable.
 */
char * read_version(int fd)
{
	static char version[CAPSELVERMAX+1];
	int __fd;

	if (fd) __fd=fd;
	else {
		__fd=open(_PATH_DEV CAPSEL_DEV, O_RDONLY);
		if (__fd<0) {
			perror("open: " _PATH_DEV CAPSEL_DEV);
			return NULL;
		}
	}
	
	if (ioctl(__fd, CAPSELVERSION, version)) {
		int error=errno;
		perror("ioctl");
		if (!fd) close(__fd);
		errno=error;
		return NULL;
	}
	
	if (!fd) close(__fd);
	version[sizeof(version)-1]='\0';
	return version;
}

int install_config(rule_list_t * list)
{
	rule_t * rule;
	char * version;
	int fd;
	int error;
	
	fd=open(_PATH_DEV CAPSEL_DEV, O_RDONLY);
	if (fd<0) {
		perror("open: " _PATH_DEV CAPSEL_DEV);
		return -errno;
	}
	
	/* first we check for capsel version */
	version=read_version(fd);
	if (!version) goto err;

	if (strcmp(version, VERSION_STRING)) {
		fprintf(stderr, "Module version (%s) mismatch my version (%s)\n",
				version, VERSION_STRING);
		errno=EINVAL;
		goto err;
	}

	/* creating new chain */
	if (ioctl(fd, CAPSELNEW, NULL)) {
		perror("ioctl: CAPSELNEW");
		goto err;
	}
	
	/* adding rules */
	for (rule=list->first_rule; rule; rule=rule->next) {
		if (ioctl(fd, CAPSELADD, rule)) {
			perror("ioctl: CAPSELADD");
			goto err;
		}
	}
	
	/* adding default rule if exists */
	if (list->default_rule) {
		if (ioctl(fd, CAPSELADDDEF, list->default_rule)) {
			perror("ioctl: CAPSELADDDEF");
			goto err;
		}
	}

	/* installing rules */
	if (ioctl(fd, CAPSELINSTALL, NULL)) {
		perror("ioctl: CAPSELINSTALL");
		goto err;
	}

	close(fd);
	return 0;
	
err:
	error=-errno;
	(void)ioctl(fd, CAPSELNEW, NULL);
	close(fd);
	return error;
}

int show_status(void)
{
	capsel_status_t status;
	int error;
	int fd;
	
	fd=open(_PATH_DEV CAPSEL_DEV, O_RDONLY);
	if (fd<0) {
		perror("open: " _PATH_DEV CAPSEL_DEV);
		return -errno;
	}
	
	/* creating new chain */
	if (ioctl(fd, CAPSELSTATUS, &status)) {
		perror("ioctl: CAPSELSTATUS");
		error = -errno;
		goto err;
	}

	close(fd);

	printf("Module status:\n");
	printf("Rules installed    = %u\n", status.count);
	printf("Configuration size = %u bytes\n", status.size);
	printf("Default rule %sinstalled\n", status.default_rule ? "": "not ");
	
	return 0;
	
err:
	error=-errno;
	close(fd);
	return error;
}

int do_ioctl(int cmd)
{
	int error;
	int fd;

	fd=open(_PATH_DEV CAPSEL_DEV, O_RDONLY);
	if (fd<0) {
		perror("open: " _PATH_DEV CAPSEL_DEV);
		return -errno;
	}
	
	if (ioctl(fd, cmd, NULL)) {
		int error=-errno;
		perror("ioctl");
		close(fd);
		return error;
	}
	
	close(fd);
	return 0;
}
