/*
 * Copyright (C) 2001,2002 Wojciech Purczynski <cliph@isec.pl>
 *
 * THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
 * IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
 * THE FREE SOFTWARE  FOUNDATION;  EITHER  VERSION 2 OF THE LICENSE, OR
 * (AT YOUR OPTION) ANY LATER VERSION.
 *
 * THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED  WARRANTY OF
 * MERCHANTABILITY OR FITNESS  FOR A PARTICULAR  PURPOSE.  SEE THE
 * GNU GENERAL PUBLIC LICENSE FOR MORE DETAILS.
 */

#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <limits.h>
#include <errno.h>
#include <string.h>

#include "config.h"
#include "conf.h"
#include "ioctl.h"

/* Valid extensions */
char * exts[] = {"cf", "cfg", "conf", "config", NULL};

/* Verifies whether the file name is correct configuration file name */
int valid_filename(char * file)
{
	char ** ext;
	
	/* must begin with alpha-numeric or underscore character */
	if (!isalnum(*file) && *file != '_')
		return 0;

	file = strrchr(file, '.');
	if (!file)
		return 1; /* OK */

	file++;

	for (ext = exts; *ext; ext++)
		if (!strcasecmp(file, *ext))
			return 1; /* OK */
	
	return 0; /* Invalid */
}

int read_and_install(int install)
{
	DIR * dir;
	struct stat st;
	int error;
	
	if (stat(CONFIG_FILE, &st)==0 && S_ISREG(st.st_mode)) {
		error=read_conf(CONFIG_FILE, 1);
		if (error) goto exit;
	}

	if (stat(CONFIG_FILE_ALT, &st)==0 && S_ISREG(st.st_mode)) {
		error=read_conf(CONFIG_FILE_ALT, 1);
		if (error) goto exit;
	}
	
	dir=opendir(CONFIG_DIR);
	if (dir) {
		struct dirent * dent;
		while ((dent=readdir(dir))!=NULL) {
			char file[PATH_MAX+1];

			if (!valid_filename(dent->d_name))
				continue;
			
			snprintf(file, sizeof(file), "%s/%s", CONFIG_DIR,
				 dent->d_name);
			
			if (!strcmp(file, CONFIG_FILE_ALT))
				continue;
			
			if (stat(file, &st)) {
				error=-errno;
				perror("stat");
				goto exit;
			}
			if (!S_ISREG(st.st_mode)) continue;
			
			error=read_conf(file, 0);
			if (error) goto exit;
		}
	}
	
	if (install) {
		error=install_config(&rules);
		if (error) goto exit;
	}
	
	error=0;
exit:
	rule_free_list(&rules);
	if (install)
		(void)remove_rules();
	return -error;
}

void usage(void)
{
	fprintf(stderr,
		NAME " " VERSION " configuration tool\n"
		"Copyright (c) 1999-2002 Wojciech Purczynski <cliph@isec.pl>\n\n"
		"Usage: " NAME " [option]\n\n"
		"Where options are:\n"
		"\t-c read and install configuration (this is default action)\n"
		"\t-t test configuration file syntax\n"
		"\t-s show module status\n"
		"\t-r removes configuration (almost disables capsel)\n"
		"\t-f flushes unused rules if previous reconfiguration has
		      not been properly finished (in case of errors)\n"
		"\t-v reads version of module\n"
		"\t-V displays my version\n");
}

int main(int argc, char * argv[])
{
	int error;
	
	if (argc==2 && argv[1][0]=='-') {
		switch (argv[1][1]) {
		
		case 'v': {
			char * version;
			version=read_version(0);
			if (!version) return errno;
			printf("Module version: %s\n", version);
			break;
		}
		
		case 'V':
			printf("Program version: %s\n", VERSION_STRING);
			break;
		
		case 'c':
			printf("Installing configuration...\n");
			error=read_and_install(1);
			return -error;

		case 't':
			printf("Parsing configuration files...\n");
			error=read_and_install(0);
			return -error;
		
		case 's':
			error=show_status();
			return -error;
		
		case 'r':
			printf("Removing configuration files...\n");
			error=remove_config();
			return -error;
		
		case 'f':
			printf("Flushing unused rules from unfinished reconfiguration...\n");
			error=remove_rules();
			return -error;
		
		default:
			usage();
		}
	} else if (argc==1) {
		int error;
		printf("Installing configuration...\n");
		error=read_and_install(1);
		return -error;
	} else {
		usage();
	}
	return 0;
}
